/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_ARBITER_HANDLER_H_
#define DBUG_ARBITER_HANDLER_H_ 1

#include <set>
#include <string>
#include <vector>

#include "dbug/arbiter/default-handler.h"
#include "dbug/arbiter/server.h"
#include "dbug/common/macros.h"
#include "dbug/common/system.h"

using dbug::thrift::Location;
using dbug::thrift::RegisterThreadReturn;

namespace dbug {
namespace arbiter {

void *handler_thread(void *args);

class Handler : public DefaultHandler {
 public:
  explicit Handler(Server *server);

  void AdvanceClock(const int tid, const int index);
  void MPIInit(const int tid,
               const int mpi_any_source,
               const int mpi_any_tag,
               const int mpi_null_process,
               const int mpi_rank);
  int MPIRank(const int dbug_rank);
  void MPIRequestComplete(const int tid, const std::string &request_id);
  int MPIReverseRank(const int dbug_rank);
  int RecordNonDeterminism(const int event);
  void RegisterThread(RegisterThreadReturn &result,  // NOLINT
                      const int parent_tid,
                      const bool create_process);
  void SetFileFlags(const int tid, const std::string &rid, const int flags);
  void SetPipeFlags(const int tid, const std::string &rid, const int flags);
  void SetPipeInput(const int tid, const std::string &rid, const int fd);
  void SetPipeOutput(const int tid, const std::string &rid, const int fd);
  void SetPipeSize(const int tid, const std::string &rid, const int size);
  void SetProcessInfo(const int tid,
                      const int pid,
                      const int ppid,
                      const int pgid,
                      const int state,
                      const int return_status,
                      const int signal);
  void SetProcessPGID(const int tid, const int pid, const int pgid);
  void SetProcessReturnStatus(const int tid,
                              const int pid,
                              const int return_status);
  void SetSocketFlags(const int tid,
                      const std::string &rid,
                      const int flags);
  void SetSocketNodeID(const int tid,
                       const std::string &rid,
                       const std::string &id);
  void SetSocketSize(const int tid,
                     const std::string &rid,
                     const int size);
  void SetThreadState(const int origin, const int target, const int state);
  void ShutdownServer();
  void UnregisterThread(const int tid);
  void UpdatePipeBytes(const int tid,
                       const std::string &rid,
                       const int nbytes);
  void UpdateSocketBytes(const int tid,
                         const std::string &rid,
                         const int nbytes);

 private:
  DISALLOW_COPY_AND_ASSIGN(Handler);
};

}  // namespace arbiter
}  // namespace dbug

#endif  // DBUG_ARBITER_HANDLER_H_
