/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_ARBITER_THREAD_H_
#define DBUG_ARBITER_THREAD_H_ 1

#include <list>
#include <string>

#include "dbug/common/clock.h"
#include "dbug/common/macros.h"

namespace dbug {
namespace arbiter {

class Process;

class Thread {
 public:
  enum State {
    RUNNING,
    DETACHED,
    JOINABLE,
    PREVIOUS,
    TERMINATED,
    WAITING
  };

  static std::string StateToString(Thread::State state);
  static int id_counter();

  Thread();

  // accessors
  Clock clock() const;
  int id() const;
  Process *pgrp() const;
  Thread::State state() const;

  // mutators
  void advance_clock(int tid);
  void join_with_clock(const Clock &clock);
  void set_clock(const Clock &clock);
  void set_pgrp(Process *process);
  void set_state(Thread::State state);

  std::list<std::string> ToString() const;

 private:
  static int id_counter_;
  Clock clock_;
  int id_;
  Thread::State old_state_;
  Process *pgrp_;
  Thread::State state_;

  DISALLOW_COPY_AND_ASSIGN(Thread);
};

}  // namespace arbiter
}  // namespace dbug

#endif  // DBUG_ARBITER_THREAD_H_
