/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_COMMON_COMMAND_H_
#define DBUG_COMMON_COMMAND_H_ 1

#include <string>
#include <vector>

#include "dbug/common/macros.h"

namespace dbug {

class Command {
 public:
  Command();

  // accessors
  std::vector<std::string> args() const;
  std::vector<std::string> binaries() const;
  std::string error_output() const;
  std::string input() const;
  int num_processes() const;
  std::string output() const;
  std::string program() const;

  // mutators
  void add_arg(std::string arg);
  void add_binary(std::string binary);
  void clear_args();
  void clear_binaries();
  void set_error_output(std::string output);
  void set_input(std::string input);
  void set_num_processes(int num_processes);
  void set_output(std::string output);
  void set_program(const std::string &program);

  std::string ToString() const;

 private:
  std::string program_;
  std::vector<std::string> args_;
  std::vector<std::string> binaries_;
  std::string input_;
  std::string output_;
  std::string error_output_;
  int num_processes_;

  DISALLOW_COPY_AND_ASSIGN(Command);
};

}  // namespace dbug

#endif  // DBUG_COMMON_COMMAND_H_
