/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_EXPLORER_EDGE_H_
#define DBUG_EXPLORER_EDGE_H_ 1

#include <list>
#include <set>
#include <string>
#include <vector>

#include "dbug/common/clock.h"
#include "dbug/common/logging.h"
#include "dbug/common/macros.h"
#include "src/thrift/explorer/explorer_types.h"

using dbug::thrift::Thread;

namespace dbug {
namespace explorer {

class Node;

class Edge {
 public:
  Edge(Node *from,
       Node *to,
       std::vector<Thread> threads,
       bool enabled,
       std::set<std::string> read_set,
       std::set<std::string> write_set);

  // accessors
  Clock clock() const;
  bool enabled() const;
  Node *from() const;
  std::set<std::string> read_set() const;
  std::vector<Thread> threads() const;
  Node *to() const;
  std::set<std::string> write_set() const;

  // mutators
  void set_read_set(const std::set<std::string> &set);
  void set_write_set(const std::set<std::string> &set);

  void Print(Logging::Verbosity verbosity) const;
  std::list<std::string> ToString() const;

 private:
  Clock clock_;
  bool enabled_;
  Node *from_;
  std::set<std::string> read_set_;
  Node *to_;
  std::vector<Thread> threads_;
  std::set<std::string> write_set_;

  DISALLOW_COPY_AND_ASSIGN(Edge);
};

}  // namespace explorer
}  // namespace dbug

#endif  // DBUG_EXPLORER_EDGE_H_

