/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_EXPLORER_HANDLER_H_
#define DBUG_EXPLORER_HANDLER_H_ 1

#include <string>
#include <vector>

#include "dbug/common/big-float.h"
#include "dbug/common/logging.h"
#include "dbug/common/macros.h"
#include "src/thrift/explorer/explorer.h"

using dbug::thrift::Move;

namespace dbug {
namespace explorer {

void *handler_thread(void *args);

class Edge;
class Node;
class Server;

class Handler : public dbug::thrift::explorerIf {
 public:
  explicit Handler(Server *server);
  ~Handler();

  void NewProcessNotification();
  void RegisterArbiter(std::vector<Move> &schedule); // NOLINT
  void UnregisterArbiter(const std::vector<dbug::thrift::Node> &branch);

 private:
  bool AreConcurrent(const Edge *edge1, const Edge *edge2) const;
  bool AreDependent(const Edge *edge1, const Edge *edge2) const;
  void ExploreEdgeFromNode(const Edge *edge, Node *node);
  void PrintComparison(Logging::Verbosity verbosity,
                       const Node *node,
                       const dbug::thrift::Node &thrift_node) const;
  void PrintThriftNode(Logging::Verbosity verbosity,
                       const dbug::thrift::Node &node) const;
  bool PossiblyCoenabled(const Edge *edge1, const Edge *edge2) const;
  void UpdateBranchProbability(Node *node, const BigFloat &delta);
  void UpdateExplorationSeeds(Node *leaf_node);
  void UpdateExplorationStatus(Node *leaf_node);
  void UpdateSizeEstimate(Node *leaf_node);

  Server *server_;

  DISALLOW_COPY_AND_ASSIGN(Handler);
};

}  // namespace explorer
}  // namespace dbug

#endif  // DBUG_EXPLORER_HANDLER_H_
