/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DBUG_INTERPOSITION_LOCAL_ARBITER_H_
#define DBUG_INTERPOSITION_LOCAL_ARBITER_H_ 1

#include <string>
#include <vector>

#include "dbug/common/macros.h"
#include "dbug/interposition/default-local-arbiter.h"
#include "src/thrift/arbiter/arbiter_types.h"

using dbug::thrift::Location;

namespace dbug {
namespace interposition {

class LocalArbiter : public DefaultLocalArbiter {
 public:
  LocalArbiter();
  virtual ~LocalArbiter();

  void CancelThreadRegistration(const int tid);
  int Choose(const int tid,
             const std::string &func,
             const std::vector<Location> &backtrace,
             const int n);
  void CompleteThreadRegistration(const int pid, const int tid);
  void PostThreadRegistration(const int tid,
                              const std::string &func,
                              const std::vector<Location> &backtrace,
                              const bool create_process,
                              int *new_pid,
                              int *new_tid);
  void ProcessSignal(const int tid,
                     const std::string &func,
                     const std::vector<Location> &backtrace,
                     const int pid,
                     const int signal);
  void ProcessTerminate(const int tid,
                        const std::string &func,
                        const std::vector<Location> &backtrace,
                        const int pid);
  void RegisterThread();
  void UnregisterAllThreads();
  void UnregisterThread();

 private:
  DISALLOW_COPY_AND_ASSIGN(LocalArbiter);
};

}  // namespace interposition
}  // namespace dbug

#endif  // DBUG_INTERPOSITION_LOCAL_ARBITER_H_
