/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "dbug/common/command.h"

#include <sstream>
#include <string>
#include <vector>

namespace dbug {

Command::Command()
    : input_(""), output_(""), error_output_(""), num_processes_(1) {}

// accessors

std::vector<std::string> Command::args() const { return args_; }

std::vector<std::string> Command::binaries() const { return binaries_; }

std::string Command::error_output() const { return error_output_; }

std::string Command::input() const { return input_; }

int Command::num_processes() const { return num_processes_; }

std::string Command::output() const { return output_; }

std::string Command::program() const { return program_; }

// mutators

void Command::add_arg(std::string arg) { args_.push_back(arg); }

void Command::add_binary(std::string binary) { binaries_.push_back(binary); }

void Command::clear_args() { args_.clear(); }

void Command::clear_binaries() { binaries_.clear(); }

void Command::set_error_output(std::string output) { error_output_ = output; }

void Command::set_input(std::string input) { input_ = input; }

void Command::set_num_processes(int n) { num_processes_ = n; }

void Command::set_output(std::string output) { output_ = output; }

void Command::set_program(const std::string &program) { program_ = program; }

// public methods

std::string Command::ToString() const {
  std::stringstream stream;
  stream << program_;
  for (unsigned i = 0; i < args_.size(); i++) { stream << " " << args_[i]; }
  if (input_ != "") { stream << " < " << input_; }
  if (output_ != "") { stream << " 1> " << output_; }
  if (error_output_ != "") { stream << " 2> " << error_output_; }
  return stream.str();
}

}  // namespace dbug
