/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <boost/scoped_ptr.hpp>

#include "dbug/common/configuration.h"
#include "dbug/common/init.h"
#include "dbug/common/system.h"
#include "dbug/common/util.h"
#include "dbug/explorer/handler.h"
#include "dbug/explorer/logging.h"
#include "dbug/explorer/server.h"

using dbug::Configuration;
using dbug::Logging;
using dbug::System;
using dbug::explorer::Server;

void usage() {
  System::Output("Usage: dbug-explorer <config_file>\n");
  System::Exit(-1);
}

int main(int argc, char *argv[]) {
  if (argc != 2) { usage(); }

  // Initialize common dbug components.
  dbug::Initialize();

  // Create a logging channel for the explorer.
  Logging::add_channel(Logging::EXPLORER,
                       NULL,
                       Configuration::explorer_verbosity());

  // Execute the exploration.
  boost::scoped_ptr<Server> server(new Server(argv[1]));
  server->Configure();
  System::Thread handler;
  System::ThreadCreate(&handler,
                       NULL,
                       dbug::explorer::handler_thread,
                       reinterpret_cast<void *>(server.get()));
  server->Explore();
  server->Stop();
  System::ThreadJoin(handler, NULL);

  // Deallocate the explorer server.
  server.reset();

  // Finalize common dbug components.
  dbug::Finalize();

  return 0;
}
