/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

// TODO(jsimsa): Implement equivalent of DefaultRemoteArbiter::DieQuietly().

#include "dbug/interposition/default-local-arbiter.h"

#include "dbug/arbiter/handler.h"
#include "dbug/arbiter/server.h"
#include "dbug/common/configuration.h"
#include "dbug/common/system.h"
#include "dbug/common/util.h"
#include "dbug/interposition/logging.h"

namespace dbug {
namespace interposition {

DefaultLocalArbiter::DefaultLocalArbiter() : Arbiter() {
  // Initialize the arbiter server.
  server_ = new dbug::arbiter::Server(Configuration::explorer_hostname(),
                                      Configuration::explorer_port(),
                                      Configuration::arbiter_port());

  if (System::GetEnvironmentVariable("DBUG_START_NTHREADS") != NULL) {
    server_->set_start_nthreads(System::StringToInt(
        System::GetEnvironmentVariable("DBUG_START_NTHREADS")));
  }
  if (System::GetEnvironmentVariable("DBUG_END_NTHREADS") != NULL) {
    server_->set_end_nthreads(System::StringToInt(
        System::GetEnvironmentVariable("DBUG_END_NTHREADS")));
  }
  if (System::GetEnvironmentVariable("DBUG_EXPLORER_PORT") != NULL) {
    server_->set_explorer_port(System::StringToInt(
        System::GetEnvironmentVariable("DBUG_EXPLORER_PORT")));
  }

  // Create a logging channel for the arbiter server.
  System::File *log_file = Util::OpenLogFile("dbug-arbiter.log");
  Logging::add_channel(Logging::ARBITER,
                       log_file,
                       Configuration::arbiter_verbosity());

  // Create a handler.
  handler_ = new dbug::arbiter::Handler(server_);

  // Register the arbiter with the explorer.
  server_->RegisterArbiter();

  // Start the manager thread.
  System::ThreadCreate(&arbiter_,
                       NULL,
                       dbug::arbiter::arbiter_thread,
                       reinterpret_cast<void *>(server_));
}

DefaultLocalArbiter::~DefaultLocalArbiter() {
  System::ThreadJoin(arbiter_, NULL);
  delete handler_;
  delete server_;
}

}  // namespace interposition
}  // namespace dbug

#include "src/interposition/gen/default-local-arbiter.cc"
