/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <mpi.h>

#include "dbug/interposition/handler.h"
#include "dbug/interposition/original.h"
#include "src/interposition/gen/mpi.cc"

namespace dbug {
namespace interposition {

#ifdef DBUG_HAVE_MPI_ABORT
int Handler::MPI_Abort(MPI_Comm comm, int errorcode) {
  return Handler::PMPI_Abort(comm, errorcode);
}
#endif

#ifdef DBUG_HAVE_MPI_ALLGATHER
int Handler::MPI_Allgather(const void *sendbuf,
                           int sendcount,
                           MPI_Datatype sendtype,
                           void *recvbuf,
                           int recvcount,
                           MPI_Datatype recvtype,
                           MPI_Comm comm) {
  return Handler::PMPI_Allgather(sendbuf,
                                 sendcount,
                                 sendtype,
                                 recvbuf,
                                 recvcount,
                                 recvtype,
                                 comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IALLGATHER
int Handler::MPI_Iallgather(const void *sendbuf,
                            int sendcount,
                            MPI_Datatype sendtype,
                            void *recvbuf,
                            int recvcount,
                            MPI_Datatype recvtype,
                            MPI_Comm comm,
                            MPI_Request *request) {
  return Handler::PMPI_Iallgather(sendbuf,
                                  sendcount,
                                  sendtype,
                                  recvbuf,
                                  recvcount,
                                  recvtype,
                                  comm,
                                  request);
}
#endif

#ifdef DBUG_HAVE_MPI_ALLGATHERV
int Handler::MPI_Allgatherv(const void *sendbuf,
                            int sendcount,
                            MPI_Datatype sendtype,
                            void *recvbuf,
                            const int *recvcounts,
                            const int *displs,
                            MPI_Datatype recvtype,
                            MPI_Comm comm) {
  return Handler::PMPI_Allgatherv(sendbuf,
                                  sendcount,
                                  sendtype,
                                  recvbuf,
                                  recvcounts,
                                  displs,
                                  recvtype,
                                  comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IALLGATHERV
int Handler::MPI_Iallgatherv(const void *sendbuf,
                             int sendcount,
                             MPI_Datatype sendtype,
                             void *recvbuf,
                             const int *recvcounts,
                             const int *displs,
                             MPI_Datatype recvtype,
                             MPI_Comm comm,
                             MPI_Request *request) {
  return Handler::PMPI_Iallgatherv(sendbuf,
                                   sendcount,
                                   sendtype,
                                   recvbuf,
                                   recvcounts,
                                   displs,
                                   recvtype,
                                   comm,
                                   request);
}
#endif

#ifdef DBUG_HAVE_MPI_ALLREDUCE
int Handler::MPI_Allreduce(const void *sendbuf,
                           void *recvbuf,
                           int count,
                           MPI_Datatype datatype,
                           MPI_Op op,
                           MPI_Comm comm) {
  return Handler::PMPI_Allreduce(sendbuf,
                                 recvbuf,
                                 count,
                                 datatype,
                                 op,
                                 comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IALLREDUCE
int Handler::MPI_Iallreduce(const void *sendbuf,
                            void *recvbuf,
                            int count,
                            MPI_Datatype datatype,
                            MPI_Op op,
                            MPI_Comm comm,
                            MPI_Request *request) {
  return Handler::PMPI_Iallreduce(sendbuf,
                                  recvbuf,
                                  count,
                                  datatype,
                                  op,
                                  comm,
                                  request);
}
#endif

#ifdef DBUG_HAVE_MPI_ALLTOALL
int Handler::MPI_Alltoall(const void *sendbuf,
                          int sendcount,
                          MPI_Datatype sendtype,
                          void *recvbuf,
                          int recvcount,
                          MPI_Datatype recvtype,
                          MPI_Comm comm) {
  return Handler::PMPI_Alltoall(sendbuf,
                                sendcount,
                                sendtype,
                                recvbuf,
                                recvcount,
                                recvtype,
                                comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IALLTOALL
int Handler::MPI_Ialltoall(const void *sendbuf,
                           int sendcount,
                           MPI_Datatype sendtype,
                           void *recvbuf,
                           int recvcount,
                           MPI_Datatype recvtype,
                           MPI_Comm comm,
                           MPI_Request *request) {
  return Handler::PMPI_Ialltoall(sendbuf,
                                 sendcount,
                                 sendtype,
                                 recvbuf,
                                 recvcount,
                                 recvtype,
                                 comm,
                                 request);
}
#endif

#ifdef DBUG_HAVE_MPI_ALLTOALLV
int Handler::MPI_Alltoallv(const void *sendbuf,
                           const int *sendcounts,
                           const int *sdispls,
                           MPI_Datatype sendtype,
                           void *recvbuf,
                           const int *recvcounts,
                           const int *rdispls,
                           MPI_Datatype recvtype,
                           MPI_Comm comm) {
  return Handler::PMPI_Alltoallv(sendbuf,
                                 sendcounts,
                                 sdispls,
                                 sendtype,
                                 recvbuf,
                                 recvcounts,
                                 rdispls,
                                 recvtype,
                                 comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IALLTOALLV
int Handler::MPI_Ialltoallv(const void *sendbuf,
                            const int *sendcounts,
                            const int *sdispls,
                            MPI_Datatype sendtype,
                            void *recvbuf,
                            const int *recvcounts,
                            const int *rdispls,
                            MPI_Datatype recvtype,
                            MPI_Comm comm,
                            MPI_Request *request) {
  return Handler::PMPI_Ialltoallv(sendbuf,
                                  sendcounts,
                                  sdispls,
                                  sendtype,
                                  recvbuf,
                                  recvcounts,
                                  rdispls,
                                  recvtype,
                                  comm,
                                  request);
}
#endif

#ifdef DBUG_HAVE_MPI_ALLTOALLW
int Handler::MPI_Alltoallw(const void *sendbuf,
                           const int *sendcounts,
                           const int *sdispls,
                           const MPI_Datatype *sendtypes,
                           void *recvbuf,
                           const int *recvcounts,
                           const int *rdispls,
                           const MPI_Datatype *recvtypes,
                           MPI_Comm comm) {
  return Handler::PMPI_Alltoallw(sendbuf,
                                 sendcounts,
                                 sdispls,
                                 sendtypes,
                                 recvbuf,
                                 recvcounts,
                                 rdispls,
                                 recvtypes,
                                 comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IALLTOALLW
int Handler::MPI_Ialltoallw(const void *sendbuf,
                            const int *sendcounts,
                            const int *sdispls,
                            const MPI_Datatype *sendtypes,
                            void *recvbuf,
                            const int *recvcounts,
                            const int *rdispls,
                            const MPI_Datatype *recvtypes,
                            MPI_Comm comm,
                            MPI_Request *request) {
  return Handler::PMPI_Ialltoallw(sendbuf,
                                  sendcounts,
                                  sdispls,
                                  sendtypes,
                                  recvbuf,
                                  recvcounts,
                                  rdispls,
                                  recvtypes,
                                  comm,
                                  request);
}
#endif

#ifdef DBUG_HAVE_MPI_BARRIER
int Handler::MPI_Barrier(MPI_Comm comm) {
  return Handler::PMPI_Barrier(comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IBARRIER
int Handler::MPI_Ibarrier(MPI_Comm comm, MPI_Request *request) {
  return Handler::PMPI_Ibarrier(comm, request);
}
#endif

#ifdef DBUG_HAVE_MPI_BCAST
int Handler::MPI_Bcast(void *buf,
                       int count,
                       MPI_Datatype datatype,
                       int root,
                       MPI_Comm comm) {
  return Handler::PMPI_Bcast(buf, count, datatype, root, comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IBCAST
int Handler::MPI_Ibcast(void *buf,
                        int count,
                        MPI_Datatype datatype,
                        int root,
                        MPI_Comm comm,
                        MPI_Request *request) {
  return Handler::PMPI_Ibcast(buf, count, datatype, root, comm, request);
}
#endif

#ifdef DBUG_HAVE_MPI_BSEND
int Handler::MPI_Bsend(const void *buf,
                       int count,
                       MPI_Datatype datatype,
                       int dest,
                       int tag,
                       MPI_Comm comm) {
  return Handler::PMPI_Bsend(buf, count, datatype, dest, tag, comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IBSEND
int Handler::MPI_Ibsend(const void *buf,
                        int count,
                        MPI_Datatype datatype,
                        int dest,
                        int tag,
                        MPI_Comm comm,
                        MPI_Request *request) {
  return Handler::PMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
}
#endif

#ifdef DBUG_HAVE_MPI_BUFFER_DETACH
int Handler::MPI_Buffer_detach(void *buffer, int *size) {
  return Handler::PMPI_Buffer_detach(buffer, size);
}
#endif

#ifdef DBUG_HAVE_MPI_CART_CREATE
int Handler::MPI_Cart_create(MPI_Comm comm,
                             int ndims,
                             const int *dims,
                             const int *periods,
                             int reorder,
                             MPI_Comm *new_comm) {
  return Handler::PMPI_Cart_create(comm,
                                   ndims,
                                   dims,
                                   periods,
                                   reorder,
                                   new_comm);
}
#endif

#ifdef DBUG_HAVE_MPI_CART_SUB
int Handler::MPI_Cart_sub(MPI_Comm comm,
                          const int *remain_dims,
                          MPI_Comm *new_comm) {
  return Handler::PMPI_Cart_sub(comm, remain_dims, new_comm);
}
#endif


#ifdef DBUG_HAVE_MPI_COMM_COMPARE
int Handler::MPI_Comm_compare(MPI_Comm comm1,
                              MPI_Comm comm2,
                              int *result) {
  return Handler::PMPI_Comm_compare(comm1, comm2, result);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_CREATE
int Handler::MPI_Comm_create(MPI_Comm comm,
                             MPI_Group group,
                             MPI_Comm *new_comm) {
  return Handler::PMPI_Comm_create(comm, group, new_comm);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_CREATE_GROUP
int Handler::MPI_Comm_create_group(MPI_Comm comm,
                                   MPI_Group group,
                                   int tag,
                                   MPI_Comm *new_comm) {
  return Handler::PMPI_Comm_create_group(comm, group, tag, new_comm);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_DUP
int Handler::MPI_Comm_dup(MPI_Comm comm, MPI_Comm *new_comm) {
  return Handler::PMPI_Comm_dup(comm, new_comm);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_DUP_WITH_INFO
int Handler::MPI_Comm_dup_with_info(MPI_Comm comm,
                                    MPI_Info info,
                                    MPI_Comm *new_comm) {
  return Handler::PMPI_Comm_dup_with_info(comm, info, new_comm);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_IDUP
int Handler::MPI_Comm_idup(MPI_Comm comm,
                           MPI_Comm *new_comm,
                           MPI_Request *request) {
  return Handler::PMPI_Comm_idup(comm, new_comm, request);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_FREE
int Handler::MPI_Comm_free(MPI_Comm *comm) {
  return Handler::PMPI_Comm_free(comm);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_GROUP
int Handler::MPI_Comm_group(MPI_Comm comm, MPI_Group *group) {
  return Handler::PMPI_Comm_group(comm, group);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_RANK
int Handler::MPI_Comm_rank(MPI_Comm comm, int *rank) {
  return Handler::PMPI_Comm_rank(comm, rank);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_SPLIT
int Handler::MPI_Comm_split(MPI_Comm comm,
                            int color,
                            int key,
                            MPI_Comm *new_comm) {
  return Handler::PMPI_Comm_split(comm, color, key, new_comm);
}
#endif

#ifdef DBUG_HAVE_MPI_COMM_SPLIT_TYPE
int Handler::MPI_Comm_split_type(MPI_Comm comm,
                                 int split_type,
                                 int key,
                                 MPI_Info info,
                                 MPI_Comm *new_comm) {
  return Handler::PMPI_Comm_split_type(comm, split_type, key, info, new_comm);
}
#endif

#ifdef DBUG_HAVE_MPI_EXSCAN
int Handler::MPI_Exscan(const void *sendbuf,
                        void *recvbuf,
                        int count,
                        MPI_Datatype datatype,
                        MPI_Op op,
                        MPI_Comm comm) {
  return Handler::PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IEXSCAN
int Handler::MPI_Iexscan(const void *sendbuf,
                         void *recvbuf,
                         int count,
                         MPI_Datatype datatype,
                         MPI_Op op,
                         MPI_Comm comm,
                         MPI_Request *request) {
  return Handler::PMPI_Iexscan(sendbuf,
                               recvbuf,
                               count,
                               datatype,
                               op,
                               comm,
                               request);
}
#endif

#ifdef DBUG_HAVE_MPI_FINALIZE
int Handler::MPI_Finalize() {
  DBUG_REQUIRE_SYMBOL(Original::MPI_Finalize,
                      MPI_FINALIZE_TYPE,
                      "MPI_Finalize");
  int ret;
  if (Interpose()) {
    Disable(NULL);
    arbiter_->UnregisterThread();
    System::MutexLock(&mpi_threads_mutex_);
    for (System::Thread tid : mpi_threads_) { System::ThreadJoin(tid, NULL); }
    System::MutexUnlock(&mpi_threads_mutex_);
    Original::MPI_Comm_free(&dbug_comm_internal_);
    Original::MPI_Comm_free(&dbug_comm_world_);
    MPI_Group_free(&dbug_group_world_);
    MPI_Group_free(&mpi_group_world_);
    ret = Original::MPI_Finalize();
    EnableFork();
    EnableCancellation();
    DEBUG_EXTERNAL("", "%d", ret);
    return ret;
  } else {
    assert(0);
  }
}
#endif

#ifdef DBUG_HAVE_MPI_GATHER
int Handler::MPI_Gather(const void *sendbuf,
                        int sendcount,
                        MPI_Datatype sendtype,
                        void *recvbuf,
                        int recvcount,
                        MPI_Datatype recvtype,
                        int root,
                        MPI_Comm comm) {
  return Handler::PMPI_Gather(sendbuf,
                              sendcount,
                              sendtype,
                              recvbuf,
                              recvcount,
                              recvtype,
                              root,
                              comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IGATHER
int Handler::MPI_Igather(const void *sendbuf,
                         int sendcount,
                         MPI_Datatype sendtype,
                         void *recvbuf,
                         int recvcount,
                         MPI_Datatype recvtype,
                         int root,
                         MPI_Comm comm,
                         MPI_Request *request) {
  return Handler::PMPI_Igather(sendbuf,
                               sendcount,
                               sendtype,
                               recvbuf,
                               recvcount,
                               recvtype,
                               root,
                               comm,
                               request);
}
#endif

#ifdef DBUG_HAVE_MPI_GATHERV
int Handler::MPI_Gatherv(const void *sendbuf,
                         int sendcount,
                         MPI_Datatype sendtype,
                         void *recvbuf,
                         const int *recvcounts,
                         const int *displs,
                         MPI_Datatype recvtype,
                         int root,
                         MPI_Comm comm) {
  return Handler::PMPI_Gatherv(sendbuf,
                               sendcount,
                               sendtype,
                               recvbuf,
                               recvcounts,
                               displs,
                               recvtype,
                               root,
                               comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IGATHERV
int Handler::MPI_Igatherv(const void *sendbuf,
                          int sendcount,
                          MPI_Datatype sendtype,
                          void *recvbuf,
                          const int *recvcounts,
                          const int *displs,
                          MPI_Datatype recvtype,
                          int root,
                          MPI_Comm comm,
                          MPI_Request *request) {
  return Handler::PMPI_Igatherv(sendbuf,
                                sendcount,
                                sendtype,
                                recvbuf,
                                recvcounts,
                                displs,
                                recvtype,
                                root,
                                comm,
                                request);
}
#endif

#ifdef DBUG_HAVE_MPI_INIT
int Handler::MPI_Init(int *argc, char ***argv) {
  return Handler::PMPI_Init(argc, argv);
}
#endif

#ifdef DBUG_HAVE_MPI_INIT_THREAD
int Handler::MPI_Init_thread(int *argc,
                             char ***argv,
                             int required,
                             int *provided) {
  return Handler::PMPI_Init_thread(argc, argv, required, provided);
}
#endif

#ifdef DBUG_HAVE_MPI_MPROBE
int Handler::MPI_Mprobe(int source,
                        int tag,
                        MPI_Comm comm,
                        MPI_Message *message,
                        MPI_Status *status) {
  return Handler::PMPI_Mprobe(source, tag, comm, message, status);
}
#endif

#ifdef DBUG_HAVE_MPI_IMPROBE
int Handler::MPI_Improbe(int source,
                         int tag,
                         MPI_Comm comm,
                         int *flag,
                         MPI_Message *message,
                         MPI_Status *status) {
  return Handler::PMPI_Improbe(source, tag, comm, flag, message, status);
}
#endif

#ifdef DBUG_HAVE_MPI_MRECV
int Handler::MPI_Mrecv(void *buf,
                       int count,
                       MPI_Datatype datatype,
                       MPI_Message *message,
                       MPI_Status *status) {
  return Handler::PMPI_Mrecv(buf, count, datatype, message, status);
}
#endif

#ifdef DBUG_HAVE_MPI_IMRECV
int Handler::MPI_Imrecv(void *buf,
                        int count,
                        MPI_Datatype datatype,
                        MPI_Message *message,
                        MPI_Request *request) {
  return Handler::PMPI_Imrecv(buf, count, datatype, message, request);
}
#endif

#ifdef DBUG_HAVE_MPI_PROBE
int Handler::MPI_Probe(int source,
                       int tag,
                       MPI_Comm comm,
                       MPI_Status *status) {
  return Handler::PMPI_Probe(source, tag, comm, status);
}
#endif

#ifdef DBUG_HAVE_MPI_IPROBE
int Handler::MPI_Iprobe(int source,
                        int tag,
                        MPI_Comm comm,
                        int *flag,
                        MPI_Status *status) {
  return Handler::PMPI_Iprobe(source, tag, comm, flag, status);
}
#endif

#ifdef DBUG_HAVE_MPI_RECV
int Handler::MPI_Recv(void *buf,
                      int count,
                      MPI_Datatype datatype,
                      int source,
                      int tag,
                      MPI_Comm comm,
                      MPI_Status *status) {
  return Handler::PMPI_Recv(buf,
                            count,
                            datatype,
                            source,
                            tag,
                            comm,
                            status);
}
#endif

#ifdef DBUG_HAVE_MPI_IRECV
int Handler::MPI_Irecv(void *buf,
                       int count,
                       MPI_Datatype datatype,
                       int source,
                       int tag,
                       MPI_Comm comm,
                       MPI_Request *request) {
  return Handler::PMPI_Irecv(buf,
                             count,
                             datatype,
                             source,
                             tag,
                             comm,
                             request);
}
#endif

#ifdef DBUG_HAVE_MPI_REDUCE
int Handler::MPI_Reduce(const void *sendbuf,
                        void *recvbuf,
                        int count,
                        MPI_Datatype datatype,
                        MPI_Op op,
                        int root,
                        MPI_Comm comm) {
  return Handler::PMPI_Reduce(sendbuf,
                              recvbuf,
                              count,
                              datatype,
                              op,
                              root,
                              comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IREDUCE
int Handler::MPI_Ireduce(const void *sendbuf,
                         void *recvbuf,
                         int count,
                         MPI_Datatype datatype,
                         MPI_Op op,
                         int root,
                         MPI_Comm comm,
                         MPI_Request *request) {
  return Handler::PMPI_Ireduce(sendbuf,
                               recvbuf,
                               count,
                               datatype,
                               op,
                               root,
                               comm,
                               request);
}
#endif

#ifdef DBUG_HAVE_MPI_REDUCE_SCATTER
int Handler::MPI_Reduce_scatter(const void *sendbuf,
                                void *recvbuf,
                                const int *recvcounts,
                                MPI_Datatype datatype,
                                MPI_Op op,
                                MPI_Comm comm) {
  return Handler::PMPI_Reduce_scatter(sendbuf,
                                      recvbuf,
                                      recvcounts,
                                      datatype,
                                      op,
                                      comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IREDUCE_SCATTER
int Handler::MPI_Ireduce_scatter(const void *sendbuf,
                                 void *recvbuf,
                                 const int *recvcounts,
                                 MPI_Datatype datatype,
                                 MPI_Op op,
                                 MPI_Comm comm,
                                 MPI_Request *request) {
  return Handler::PMPI_Ireduce_scatter(sendbuf,
                                       recvbuf,
                                       recvcounts,
                                       datatype,
                                       op,
                                       comm,
                                       request);
}
#endif

#ifdef DBUG_HAVE_MPI_REDUCE_SCATTER_BLOCK
int Handler::MPI_Reduce_scatter_block(const void *sendbuf,
                                      void *recvbuf,
                                      int recvcount,
                                      MPI_Datatype datatype,
                                      MPI_Op op,
                                      MPI_Comm comm) {
  return Handler::PMPI_Reduce_scatter_block(sendbuf,
                                            recvbuf,
                                            recvcount,
                                            datatype,
                                            op,
                                            comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IREDUCE_SCATTER_BLOCK
int Handler::MPI_Ireduce_scatter_block(const void *sendbuf,
                                       void *recvbuf,
                                       int recvcount,
                                       MPI_Datatype datatype,
                                       MPI_Op op,
                                       MPI_Comm comm,
                                       MPI_Request *request) {
  return Handler::PMPI_Ireduce_scatter_block(sendbuf,
                                             recvbuf,
                                             recvcount,
                                             datatype,
                                             op,
                                             comm,
                                             request);
}
#endif

#ifdef DBUG_HAVE_MPI_REQUEST_FREE
int Handler::MPI_Request_free(MPI_Request *request) {
  return Handler::PMPI_Request_free(request);
}
#endif

#ifdef DBUG_HAVE_MPI_RSEND
int Handler::MPI_Rsend(const void *buf,
                       int count,
                       MPI_Datatype datatype,
                       int dest,
                       int tag,
                       MPI_Comm comm) {
  return Handler::PMPI_Rsend(buf, count, datatype, dest, tag, comm);
}
#endif

#ifdef DBUG_HAVE_MPI_IRSEND
int Handler::MPI_Irsend(const void *buf,
                        int count,
                        MPI_Datatype datatype,
                        int dest,
                        int tag,
                        MPI_Comm comm,
                        MPI_Request *request) {
  return Handler::PMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
}
#endif

#ifdef DBUG_HAVE_MPI_SCAN
int Handler::MPI_Scan(const void *sendbuf,
                      void *recvbuf,
                      int count,
                      MPI_Datatype datatype,
                      MPI_Op op,
                      MPI_Comm comm) {
  return Handler::PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
}
#endif

#ifdef DBUG_HAVE_MPI_ISCAN
int Handler::MPI_Iscan(const void *sendbuf,
                       void *recvbuf,
                       int count,
                       MPI_Datatype datatype,
                       MPI_Op op,
                       MPI_Comm comm,
                       MPI_Request *request) {
  return Handler::PMPI_Iscan(sendbuf,
                             recvbuf,
                             count,
                             datatype,
                             op,
                             comm,
                             request);
}
#endif

#ifdef DBUG_HAVE_MPI_SCATTER
int Handler::MPI_Scatter(const void *sendbuf,
                         int sendcount,
                         MPI_Datatype sendtype,
                         void *recvbuf,
                         int recvcount,
                         MPI_Datatype recvtype,
                         int root,
                         MPI_Comm comm) {
  return Handler::PMPI_Scatter(sendbuf,
                               sendcount,
                               sendtype,
                               recvbuf,
                               recvcount,
                               recvtype,
                               root,
                               comm);
}
#endif

#ifdef DBUG_HAVE_MPI_ISCATTER
int Handler::MPI_Iscatter(const void *sendbuf,
                          int sendcount,
                          MPI_Datatype sendtype,
                          void *recvbuf,
                          int recvcount,
                          MPI_Datatype recvtype,
                          int root,
                          MPI_Comm comm,
                          MPI_Request *request) {
  return Handler::PMPI_Iscatter(sendbuf,
                                sendcount,
                                sendtype,
                                recvbuf,
                                recvcount,
                                recvtype,
                                root,
                                comm,
                                request);
}
#endif

#ifdef DBUG_HAVE_MPI_SCATTERV
int Handler::MPI_Scatterv(const void *sendbuf,
                          const int *sendcounts,
                          const int *displs,
                          MPI_Datatype sendtype,
                          void *recvbuf,
                          int recvcount,
                          MPI_Datatype recvtype,
                          int root,
                          MPI_Comm comm) {
  return Handler::PMPI_Scatterv(sendbuf,
                                sendcounts,
                                displs,
                                sendtype,
                                recvbuf,
                                recvcount,
                                recvtype,
                                root,
                                comm);
}
#endif

#ifdef DBUG_HAVE_MPI_ISCATTERV
int Handler::MPI_Iscatterv(const void *sendbuf,
                           const int *sendcounts,
                           const int *displs,
                           MPI_Datatype sendtype,
                           void *recvbuf,
                           int recvcount,
                           MPI_Datatype recvtype,
                           int root,
                           MPI_Comm comm,
                           MPI_Request *request) {
  return Handler::PMPI_Iscatterv(sendbuf,
                                 sendcounts,
                                 displs,
                                 sendtype,
                                 recvbuf,
                                 recvcount,
                                 recvtype,
                                 root,
                                 comm,
                                 request);
}
#endif

#ifdef DBUG_HAVE_MPI_SEND
int Handler::MPI_Send(const void *buf,
                      int count,
                      MPI_Datatype datatype,
                      int dest,
                      int tag,
                      MPI_Comm comm) {
  return Handler::PMPI_Send(buf, count, datatype, dest, tag, comm);
}
#endif

#ifdef DBUG_HAVE_MPI_ISEND
int Handler::MPI_Isend(const void *buf,
                       int count,
                       MPI_Datatype datatype,
                       int dest,
                       int tag,
                       MPI_Comm comm,
                       MPI_Request *request) {
  return Handler::PMPI_Isend(buf, count, datatype, dest, tag, comm, request);
}
#endif

#ifdef DBUG_HAVE_MPI_SENDRECV
int Handler::MPI_Sendrecv(const void *sendbuf,
                          int sendcount,
                          MPI_Datatype sendtype,
                          int dest,
                          int sendtag,
                          void *recvbuf,
                          int recvcount,
                          MPI_Datatype recvtype,
                          int source,
                          int recvtag,
                          MPI_Comm comm,
                          MPI_Status *status) {
  return Handler::PMPI_Sendrecv(sendbuf,
                                sendcount,
                                sendtype,
                                dest,
                                sendtag,
                                recvbuf,
                                recvcount,
                                recvtype,
                                source,
                                recvtag,
                                comm,
                                status);
}
#endif

#ifdef DBUG_HAVE_MPI_SENDRECV_REPLACE
int Handler::MPI_Sendrecv_replace(void *buf,
                                  int count,
                                  MPI_Datatype datatype,
                                  int dest,
                                  int sendtag,
                                  int source,
                                  int recvtag,
                                  MPI_Comm comm,
                                  MPI_Status *status) {
  return Handler::PMPI_Sendrecv_replace(buf,
                                        count,
                                        datatype,
                                        dest,
                                        sendtag,
                                        source,
                                        recvtag,
                                        comm,
                                        status);
}
#endif

#ifdef DBUG_HAVE_MPI_SSEND
int Handler::MPI_Ssend(const void *buf,
                       int count,
                       MPI_Datatype datatype,
                       int dest,
                       int tag,
                       MPI_Comm comm) {
  return Handler::PMPI_Ssend(buf, count, datatype, dest, tag, comm);
}
#endif

#ifdef DBUG_HAVE_MPI_ISSEND
int Handler::MPI_Issend(const void *buf,
                        int count,
                        MPI_Datatype datatype,
                        int dest,
                        int tag,
                        MPI_Comm comm,
                        MPI_Request *request) {
  return Handler::PMPI_Issend(buf, count, datatype, dest, tag, comm, request);
}
#endif

#ifdef DBUG_HAVE_MPI_TEST
int Handler::MPI_Test(MPI_Request *request,
                      int *flag,
                      MPI_Status *status) {
  return Handler::PMPI_Test(request, flag, status);
}
#endif

#ifdef DBUG_HAVE_MPI_TESTALL
int Handler::MPI_Testall(int count,
                         MPI_Request *requests,
                         int *flag,
                         MPI_Status *statuses) {
  return Handler::PMPI_Testall(count, requests, flag, statuses);
}
#endif

#ifdef DBUG_HAVE_MPI_TESTANY
int Handler::MPI_Testany(int count,
                         MPI_Request *requests,
                         int *index,
                         int *flag,
                         MPI_Status *status) {
  return Handler::PMPI_Testany(count, requests, index, flag, status);
}
#endif

#ifdef DBUG_HAVE_MPI_TESTSOME
int Handler::MPI_Testsome(int incount,
                          MPI_Request *requests,
                          int *outcount,
                          int *indices,
                          MPI_Status *statuses) {
  return Handler::PMPI_Testsome(incount, requests, outcount, indices, statuses);
}
#endif

#ifdef DBUG_HAVE_MPI_WAIT
int Handler::MPI_Wait(MPI_Request *request,
                      MPI_Status *status) {
  return Handler::PMPI_Wait(request, status);
}
#endif

#ifdef DBUG_HAVE_MPI_WAITALL
int Handler::MPI_Waitall(int count,
                         MPI_Request *requests,
                         MPI_Status *statuses) {
  return Handler::PMPI_Waitall(count, requests, statuses);
}
#endif

#ifdef DBUG_HAVE_MPI_WAITANY
int Handler::MPI_Waitany(int count,
                         MPI_Request *requests,
                         int *index,
                         MPI_Status *status) {
  return Handler::PMPI_Waitany(count, requests, index, status);
}
#endif

#ifdef DBUG_HAVE_MPI_WAITSOME
int Handler::MPI_Waitsome(int incount,
                          MPI_Request *requests,
                          int *outcount,
                          int *indices,
                          MPI_Status *statuses) {
  return Handler::PMPI_Waitsome(incount, requests, outcount, indices, statuses);
}
#endif

}  // namespace interposition
}  // namespace dbug
