/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <mpi.h>
#include <pthread.h>
#include <string.h>

#define BUFFER_SIZE 256

char message1[BUFFER_SIZE], message2[BUFFER_SIZE];

void *slave_thread(void * /* args */) {
  int myrank;
  MPI_Status status;
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  if (myrank == 0) {
    MPI_Recv(message2, BUFFER_SIZE, MPI_CHAR, 1, 99, MPI_COMM_WORLD, &status);
    printf("Process %d received '%s'.\n", myrank, message2);
  } else if (myrank == 1) {
    MPI_Recv(message1, BUFFER_SIZE, MPI_CHAR, 0, 99, MPI_COMM_WORLD, &status);
    printf("Process %d received '%s'.\n", myrank, message1);
  }
  return NULL;
}

int main(int argc, char *argv[]) {
  int provided;
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &provided);
  pthread_t tid;
  assert(pthread_create(&tid, NULL, slave_thread, NULL) == 0);
  int myrank;
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  if (myrank == 0) {
    /* code for process zero */
    snprintf(message1, BUFFER_SIZE, "Message A");
    MPI_Send(message1, strlen(message1) + 1, MPI_CHAR, 1, 99, MPI_COMM_WORLD);
  } else if (myrank == 1) {
    /* code for process one */
    snprintf(message2, BUFFER_SIZE, "Message B");
    MPI_Send(message2, strlen(message2) + 1, MPI_CHAR, 0, 99, MPI_COMM_WORLD);
  }
  assert(pthread_join(tid, NULL) == 0);
  MPI_Finalize();
  return 0;
}
