/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <mpi.h>
#include <stdio.h>
#include <string.h>

#include <boost/scoped_array.hpp>

#define BUFFER_SIZE 256

int main(int argc, char *argv[]) {
  char buffer[BUFFER_SIZE];
  MPI_Init(&argc, &argv);
  int myrank;
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  int size;
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  assert(size == 5);
  if (myrank == 0) {
    while (1) {
      int flag;
      MPI_Status status;
      MPI_Iprobe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &flag, &status);
      if (!flag) { break; }
      MPI_Status status2;
      MPI_Recv(buffer, BUFFER_SIZE, MPI_CHAR,
               status.MPI_SOURCE, status.MPI_TAG,
               MPI_COMM_WORLD, &status2);
      printf("Process %d received '%s'.\n", myrank, buffer);
    }
  }
  if (myrank == 1) {
    int size;
    MPI_Pack_size(BUFFER_SIZE, MPI_CHAR, MPI_COMM_WORLD, &size);
    int bufsize = MPI_BSEND_OVERHEAD + size;
    boost::scoped_array<char> buf(new char[bufsize]);
    MPI_Buffer_attach(buf.get(), bufsize);
    snprintf(buffer, BUFFER_SIZE, "Message A");
    MPI_Bsend(buffer, strlen(buffer) + 1, MPI_CHAR, 0, myrank, MPI_COMM_WORLD);
    int idummy;
    char *dummy;
    MPI_Buffer_detach(&dummy, &idummy);
  }
  if (myrank == 2) {
    snprintf(buffer, BUFFER_SIZE, "Message B");
    MPI_Rsend(buffer, strlen(buffer) + 1, MPI_CHAR, 0, myrank, MPI_COMM_WORLD);
  }
  if (myrank == 3) {
    snprintf(buffer, BUFFER_SIZE, "Message C");
    MPI_Send(buffer, strlen(buffer) + 1, MPI_CHAR, 0, myrank, MPI_COMM_WORLD);
  }
  if (myrank == 4) {
    snprintf(buffer, BUFFER_SIZE, "Message D");
    MPI_Ssend(buffer, strlen(buffer) + 1, MPI_CHAR, 0, myrank, MPI_COMM_WORLD);
  }
  MPI_Finalize();
  return 0;
}
