/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <mpi.h>
#include <stdio.h>

int main(int argc, char *argv[]) {
  int myrank, size;
  MPI_Status status;
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  assert(size == 3);
  if (myrank == 0) {
    int i = 1;
    MPI_Send(&i, 1, MPI_INT, 2, 99, MPI_COMM_WORLD);
  }
  if (myrank == 1) {
    double d = 3.14;
    MPI_Send(&d, 1, MPI_DOUBLE, 2, 99, MPI_COMM_WORLD);
  }
  if (myrank == 2) {
    for (int i = 0; i < 2; i++) {
      MPI_Probe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
      printf("Probe matched %lld bytes from source %d.\n",
             status.count, status.MPI_SOURCE);
      if (status.MPI_SOURCE == 0) {
        int i;
        MPI_Recv(&i, 1, MPI_INT, 0, 99, MPI_COMM_WORLD, &status);
        printf("Process %d received %d from %d.\n",
               myrank, i, status.MPI_SOURCE);
      }
      if (status.MPI_SOURCE == 1) {
        double d;
        MPI_Recv(&d, 1, MPI_DOUBLE, 1, 99, MPI_COMM_WORLD, &status);
        printf("Process %d received %f from %d.\n",
               myrank, d, status.MPI_SOURCE);
      }
    }
  }
  MPI_Finalize();
  return 0;
}
