/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>

pthread_mutex_t mutex;
pthread_mutex_t rec_mutex;

void *regular_thread(void * /* args */) {
  assert(pthread_mutex_lock(&mutex) == 0);
  assert(pthread_mutex_destroy(&mutex) == EBUSY);
  assert(pthread_mutex_lock(&mutex) == EDEADLK);
  assert(pthread_mutex_trylock(&mutex) == EDEADLK);
  printf("Executing regular child thread.\n");
  assert(pthread_mutex_unlock(&mutex) == 0);
  return NULL;
}

void *recursive_thread(void * /* args */) {
  assert(pthread_mutex_lock(&rec_mutex) == 0);
  assert(pthread_mutex_lock(&rec_mutex) == 0);
  assert(pthread_mutex_destroy(&rec_mutex) == EBUSY);
  printf("Executing recursive child thread.\n");
  assert(pthread_mutex_unlock(&rec_mutex) == 0);
  assert(pthread_mutex_unlock(&rec_mutex) == 0);
  return NULL;
}

void *trylock_thread(void *args) {
  assert(pthread_mutex_unlock(
      reinterpret_cast<pthread_mutex_t *>(args)) == EPERM);
  assert(pthread_mutex_trylock(
      reinterpret_cast<pthread_mutex_t *>(args)) == EBUSY);
  return NULL;
}

int main() {
  pthread_t tid1, tid2;
  // assert(pthread_mutex_lock(&mutex) == EINVAL);
  // assert(pthread_mutex_trylock(&mutex) == EINVAL);
  // assert(pthread_mutex_unlock(&mutex) == EINVAL);

  assert(pthread_mutex_init(&mutex, NULL) == 0);
  assert(pthread_mutex_init(&mutex, NULL) == EBUSY);
  assert(pthread_mutex_unlock(&mutex) == EPERM);

  assert(pthread_create(&tid1, NULL, regular_thread, NULL) == 0);
  assert(pthread_mutex_lock(&mutex) == 0);
  assert(pthread_create(&tid2, NULL, trylock_thread, &mutex) == 0);
  printf("Executing regular parent thread.\n");
  assert(pthread_join(tid2, NULL) == 0);
  assert(pthread_mutex_unlock(&mutex) == 0);
  assert(pthread_join(tid1, NULL) == 0);
  assert(pthread_mutex_trylock(&mutex) == 0);
  assert(pthread_mutex_unlock(&mutex) == 0);
  assert(pthread_mutex_destroy(&mutex) == 0);
  assert(pthread_mutex_destroy(&mutex) == EINVAL);

  pthread_mutexattr_t attr;
  assert(pthread_mutexattr_init(&attr) == 0);
  assert(pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE) == 0);
  assert(pthread_mutex_init(&rec_mutex, &attr) == 0);
  assert(pthread_create(&tid1, NULL, recursive_thread, NULL) == 0);
  assert(pthread_mutex_lock(&rec_mutex) == 0);
  assert(pthread_mutex_lock(&rec_mutex) == 0);
  assert(pthread_create(&tid2, NULL, trylock_thread, &rec_mutex) == 0);
  printf("Executing recursive parent thread.\n");
  assert(pthread_join(tid2, NULL) == 0);
  assert(pthread_mutex_unlock(&rec_mutex) == 0);
  assert(pthread_mutex_unlock(&rec_mutex) == 0);
  assert(pthread_join(tid1, NULL) == 0);
  assert(pthread_mutex_trylock(&rec_mutex) == 0);
  assert(pthread_mutex_trylock(&rec_mutex) == 0);
  assert(pthread_mutex_unlock(&rec_mutex) == 0);
  assert(pthread_mutex_unlock(&rec_mutex) == 0);
  assert(pthread_mutex_destroy(&rec_mutex) == 0);
  assert(pthread_mutex_destroy(&rec_mutex) == EINVAL);
  return 0;
}
