/*
 * Authors: Jiri Simsa
 * Copyright (c) 2013, Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>

pthread_spinlock_t spin;

void *lock_thread(void * /* args */) {
  assert(pthread_spin_lock(&spin) == 0);
  assert(pthread_spin_destroy(&spin) == EBUSY);
  assert(pthread_spin_lock(&spin) == EDEADLK);
  assert(pthread_spin_trylock(&spin) == EDEADLK);
  printf("Executing child thread.\n");
  assert(pthread_spin_unlock(&spin) == 0);
  return NULL;
}

void *trylock_thread(void * /* args */) {
  assert(pthread_spin_unlock(&spin) == EPERM);
  assert(pthread_spin_trylock(&spin) == EBUSY);
  return NULL;
}

int main() {
  pthread_t tid1, tid2;
  assert(pthread_spin_lock(&spin) == EINVAL);
  assert(pthread_spin_trylock(&spin) == EINVAL);
  assert(pthread_spin_unlock(&spin) == EINVAL);

  assert(pthread_spin_init(&spin, 0) == 0);
  assert(pthread_spin_init(&spin, 0) == EBUSY);
  assert(pthread_spin_unlock(&spin) == EPERM);

  assert(pthread_create(&tid1, NULL, lock_thread, NULL) == 0);
  assert(pthread_spin_lock(&spin) == 0);
  assert(pthread_create(&tid2, NULL, trylock_thread, NULL) == 0);
  printf("Executing parent thread.\n");
  assert(pthread_join(tid2, NULL) == 0);
  assert(pthread_spin_unlock(&spin) == 0);
  assert(pthread_join(tid1, NULL) == 0);

  assert(pthread_spin_trylock(&spin) == 0);
  assert(pthread_spin_unlock(&spin) == 0);

  assert(pthread_spin_destroy(&spin) == 0);
  assert(pthread_spin_destroy(&spin) == EINVAL);
  return 0;
}
